-- Overscan 0.25 - (c) 2005-2018 Martin Breidt (martin@breidt.net)
-- Useage: Run this script with an active camera view.
--
--
-- This code is released under "Quote ware" license:
--      If you use this tool in a production environment with a group of more than two people,
--      or have used it in the past under such conditions, then you are obliged to tell 
--      me (martin@breidt.net) about it and allow me to list that project title and your 
--      company name as a reference on my website http://scripts.breidt.net
--
-- Adjust render image width and height and the script will adjust
-- render resolution and camera focal length so that your previous
-- camera view will be retained, but zoomed in/out
-- This can be useful if you need to render additional pixels to the
-- sides of the image without changing the perspective
--
-- The conversion forumla is based on a webforum posting by Graham Calhoun
--
-- New in 0.2: Option to keep original camera; old resolution is stored; relative values for sizes
-- New in 0.25: Added compatibility with Vray Physical Camera, updated UI by lee johnson (lj@jtwo.com)

rollout _roOverScan "New Image Dimensions" 
(
	groupbox _gbRnNfo "Render Info" pos:[5,5] width:178 height:60
		label _lblOldSz "Original Render Size: " across:2 align:#left offset:[0,-50]
		label _lblStartRes "" offset:[16,-50]
		
		button _btnResetRes "Refresh Render Size" tooltip:"Use this if render size changed in Render Settings." width:164
	
	groupbox _gbOsStng "Overscan Settings" pos:[5,70] width:178 height:77
--		checkbox _chkbxIncBy "" checked:false align:#left  across:2 offset:[-1,-67] tooltip:"Input increased pixel amount instead of new render size."
--		label _lblIncBy "Relative Overscan Amount" offset:[-39,-67] tooltip:"Input increased pixel amount instead of new render size."
	
		checkbox _chkbxIncBy "    Relative Overscan" checked:false align:#left  offset:[0,-67] tooltip:"Input increased pixel amount instead of new render size."
	
		spinner _spnrXresNew "New Width:      " range:[-10000,10000,1] type:#integer width:135 align:#left across:2
		label _xdim "pixels" offset:[18,0]
	
		spinner _spnrYresNew "New Height:     " range:[-10000,10000,1] type:#integer width:135 align:#left across:2
		label _ydim "pixels" offset:[18,0]
	
	groupbox _gbNewCam "Camera Operations" pos:[5,153] width:178 height:63
--		checkbox _chkbxKeepOrigCam "" checked:false align:#left  across:2 offset:[-1,-53] tooltip:"Keep original camera and create new renamed camera with new render size."
--		label _lblKeepOrigCam "Keep Original Camera as Copy" offset:[-31,-53] tooltip:"Keep original camera and create new renamed camera with new render size."
	
		checkbox _chkbxKeepOrigCam "    Copy Original Camera" checked:false align:#left  offset:[0,-53] tooltip:"Keep original camera and create new renamed camera with new render size."
	
		button _btnAppOvrScn " Apply Overscan" tooltip:"Modify/create camera and adjust render settings to match desired resolution. Active viewport has to be a camera view." width:164

	local currentXres, currentYres		-- the currently used resolution

	fn camUsesAdvancedPerspective theCam = (		-- return false if the provided camera uses any advanced perspective values, such as Perspective Control or Lens Distortion
		local advProps = #( DataPair #zoom_factor 1.0, DataPair #lens_breathing_amount 1.0, DataPair #horizontal_shift 0.0, DataPair #vertical_shift 0.0, DataPair #horizontal_tilt_correction 0.0, DataPair #vertical_tilt_correction 0.0, DataPair #auto_vertical_tilt_correction false, DataPair #distortion_type 0)		-- array of properties and their neutral values
		
		local res = false
		if (superClassOf theCam)==camera then 
		(
			-- check whether each advanced property is at default values
			for p in advProps where (isProperty theCam p.v1) while not res do 
			(
				res = (getProperty theCam p.v1)!=p.v2
			)
		) else (
			res = true
			format "Not a camera!\n"
		)
		res
	)
	
	on _btnResetRes pressed do 	(
		_chkbxIncBy.checked = false
		_spnrXresNew.value = currentXres = renderWidth
		_spnrYresNew.value = currentYres = renderHeight
		_lblStartRes.text = ( (currentXres as string) + " x " + (currentYres as string) )  
	)

	on _chkbxIncBy changed state do	(
		if state then 
		(	-- show size deltas
			_spnrXresNew.value -= currentXres
			_spnrYresNew.value -= currentYres
			_spnrXresNew.text = "Change Width: "
			_spnrYresNew.text = "Change Height:"
		) 
		else 
		(	-- show absolute width/height
			_spnrXresNew.value += currentXres
			_spnrYresNew.value += currentYres
			_spnrXresNew.text = "New Width:      "
			_spnrYresNew.text = "New Height:     "
		)
	)

	on _btnAppOvrScn pressed do (
		local cam = getActiveCamera()
		
		if (cam!=undefined) then (
			undo "Overscan" on (	
					renderSceneDialog.close()				-- need to close dialog to make changes stick
					
					local applyChange = true
					
					if ((classOf cam) == VRayPhysicalCamera) or ((classOf cam)==Physical) then 
					(
						if (camUsesAdvancedPerspective cam) then 
						(	-- Issue warning when advanced camera features (Perspective Control, Lens Distortion) are enabled
							applyChange = queryBox "This camera uses advanced perspective controls (such as Lens Shift, Tilt Control, Lens Distortion) which cannot be correctly handled by Overscan.\n\nContinue anyway?" title:"Overscan Warning"
						)
					)
					
					if applyChange then 
					(
						-- create copy of camera
						local newCam
						if _chkbxKeepOrigCam.checked then 
						(
							-- create copy of camera
							if not (maxOps.cloneNodes cam newNodes:&newCam) then throw "Overscan: error! cannot copy camera"
							newCam[1].name = (cam.name + "_overscan" + (_spnrXresNew.value as string) + "x" + (_spnrYresNew.value as string))
							cam = newCam[1]
						)
						local xres = _spnrXresNew.value
						local yres = _spnrYresNew.value
						
						if _chkbxIncBy.checked then 
						(
							xres += currentXres
							yres += currentYres
						)
				
						if ((classOf cam) == VRayPhysicalCamera) or ((classOf cam)==Physical) then 
						(
							cam.specify_fov = true 		-- LJ: this code line written by ScriptSpot User K2000
						)
						
						local aw_h = getRendApertureWidth() * 0.5
						local old_f = aw_h / (tan (cam.fov*0.5))
						local new_f = old_f * (renderWidth/(xres as float))
						cam.fov = 2 * atan (aw_h / new_f)
						-- set new render resolution
						renderWidth = xres		-- no undo possible
						renderHeight = yres		-- no undo possible
						viewport.setCamera cam
						_btnResetRes.pressed()
					) else messageBox "No changes were made." beep:false title:"Overscan"
				)
				
		)
		else (
			messageBox "Active viewport is not a camera viewport." title:"Overscan Error"
		)
	)
		
	on _roOverScan open do (
		_btnResetRes.pressed()
	)
)

rollout _roAbout "About v0.25"
(
	label _02l0 " 2005-2018 M. Breidt"
	label _02l1 "This tool takes the camera from" align:#left
	label _02l2 "the currently active viewport and" align:#left
	label _02l3 "adjusts the focal length such that" align:#left
	label _02l35 "when rendering a new image with" align:#left
	label _02l4 "new width/height, the old image" align:#left
	label _02l5 "will be a sub-/superimage of the" align:#left
	label _02l6 "new without changing the per-" align:#left
	label _02l7 "spective. This is useful, if you" align:#left
	label _02l8 "need extra space at the edges" align:#left
	label _02l9 " of your image." align:#left
	label _0220 " "
	label _03l0 "April 2018 - v 0.25"
	label _03l1 "Updated to add compatibility with" align:#left
	label _03l2 "Vray Physical Camera, by Script-" align:#left
	label _03l3 "spot user K2000 and streamlined" align:#left
	label _03l4 "UI by Lee Johnson - lj@jtwo.com" align:#left
)

-- close existing rolloutFloater
try (closeRolloutFloater overscanDialog) catch ()
overscanDialog = newRolloutFloater "Overscan 0.25" 200 270
addRollout _roOverScan overscanDialog
addRollout _roAbout overscanDialog rolledUp:true
